@@ -1,41 +1,21 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from sql import Literal
-from sql.aggregate import Max, Min, Sum
-from sql.functions import Position
-from sql.operators import Concat
+import stdnum.exceptions
+from sql.aggregate import Min, Sum
+
+try:
+    from stdnum.be import ogm_vcs
+except ImportError:
+    from . import ogm_vcs
 
 from trytond.model import ModelSQL, ModelView, fields
+from trytond.modules.account.exceptions import FiscalYearNotFoundError
 from trytond.modules.currency.fields import Monetary
 from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
 
 
-class AccountTemplate(metaclass=PoolMeta):
-    __name__ = 'account.account.template'
-
-    @classmethod
-    def __register__(cls, module_name):
-        pool = Pool()
-        ModelData = pool.get('ir.model.data')
-        cursor = Transaction().connection.cursor()
-        model_data = ModelData.__table__()
-
-        # Migration from 3.4: translation of the account chart
-        cursor.execute(*model_data.select(model_data.id,
-                where=((model_data.fs_id == 'be')
-                    & (model_data.module == 'account_be'))))
-        if cursor.fetchone():
-            cursor.execute(*model_data.update(
-                    columns=[model_data.fs_id],
-                    values=[Concat(model_data.fs_id, '_fr')],
-                    where=((Position('_fr', model_data.fs_id) == 0)
-                        & (model_data.module == 'account_be'))))
-
-        super(AccountTemplate, cls).__register__(module_name)
-
-
 class CreateChart(metaclass=PoolMeta):
     __name__ = 'account.create_chart'
 
@@ -58,7 +38,6 @@ class CreateChart(metaclass=PoolMeta):
 
 
 class BEVATCustomer(ModelSQL, ModelView):
-    "Belgium VAT Customer"
     __name__ = 'account.be.vat_customer'
 
     company_tax_identifier = fields.Many2One(
@@ -124,11 +103,7 @@ class BEVATCustomer(ModelSQL, ModelView):
             .join(party_identifier,
                 condition=invoice.party_tax_identifier == party_identifier.id)
             .select(
-                Max(invoice_tax.id).as_('id'),
-                Literal(0).as_('create_uid'),
-                Min(invoice_tax.create_date).as_('create_date'),
-                Literal(0).as_('write_uid'),
-                Max(invoice_tax.write_date).as_('write_date'),
+                Min(invoice_tax.id).as_('id'),
                 invoice.tax_identifier.as_('company_tax_identifier'),
                 invoice.party_tax_identifier.as_('party_tax_identifier'),
                 Sum(invoice_tax.base).as_('turnover'),
@@ -143,14 +118,13 @@ class BEVATCustomer(ModelSQL, ModelView):
 
 
 class BEVATCustomerContext(ModelView):
-    "Belgium VAT Customer Context"
     __name__ = 'account.be.vat_customer.context'
 
     company = fields.Many2One('company.company', "Company", required=True)
     fiscalyear = fields.Many2One(
         'account.fiscalyear', "Fiscal Year", required=True,
         domain=[
-            ('company', '=', Eval('company')),
+            ('company', '=', Eval('company', -1)),
             ])
 
     @classmethod
@@ -162,6 +136,82 @@ class BEVATCustomerContext(ModelView):
         pool = Pool()
         FiscalYear = pool.get('account.fiscalyear')
         context = Transaction().context
-        return context.get(
-            'fiscalyear',
-            FiscalYear.find(context.get('company'), exception=False))
+        if 'fiscalyear' not in context:
+            try:
+                fiscalyear = FiscalYear.find(
+                    cls.default_company(), test_state=False)
+            except FiscalYearNotFoundError:
+                return None
+            return fiscalyear.id
+        return context['fiscalyear']
+
+
+class Invoice(metaclass=PoolMeta):
+    __name__ = 'account.invoice'
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.supplier_payment_reference_type.selection.append(
+            ('ogm_vcs', "Belgian structured communication"))
+
+    @classmethod
+    def _format_supplier_payment_reference_ogm_vcs(cls, reference):
+        try:
+            return ogm_vcs.format(reference)
+        except stdnum.exceptions.ValidationError:
+            return reference
+
+    @classmethod
+    def _search_customer_payment_reference(cls, value):
+        yield from super()._search_customer_payment_reference(value)
+        if ogm_vcs.is_valid(value):
+            yield ('number_digit', '=', int(ogm_vcs.compact(value)[:-2]))
+
+    def _customer_payment_reference_type(self):
+        type = super()._customer_payment_reference_type()
+        if (self.invoice_address.country
+                and self.invoice_address.country.code == 'BE'):
+            type = 'ogm_vcs'
+        return type
+
+    def _format_customer_payment_reference_ogm_vcs(self, source):
+        number = self._customer_payment_reference_number_digit(source)
+        if number:
+            number = number % 10**10
+            check_digit = ogm_vcs.calc_check_digit(str(number))
+            reference = f'{self.number_digit:0>10}{check_digit}'
+            return f'+++{ogm_vcs.format(reference)}+++'
+
+    def _check_supplier_payment_reference_ogm_vcs(self):
+        return ogm_vcs.is_valid(self.supplier_payment_reference)
+
+
+class Payment(metaclass=PoolMeta):
+    __name__ = 'account.payment'
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.reference_type.selection.append(
+            ('ogm_vcs', "Belgian structured communication"))
+
+    @classmethod
+    def _format_reference_ogm_vcs(cls, reference):
+        try:
+            return ogm_vcs.format(reference)
+        except stdnum.exceptions.ValidationError:
+            return reference
+
+    def _check_reference_ogm_vcs(self):
+        return ogm_vcs.is_valid(self.reference)
+
+
+class StatementRuleLine(metaclass=PoolMeta):
+    __name__ = 'account.statement.rule.line'
+
+    @classmethod
+    def _party_payment_reference(cls, value):
+        yield from super()._party_payment_reference(value)
+        if ogm_vcs.is_valid(value):
+            yield ('code_digit', '=', int(ogm_vcs.compact(value)[:-2]))
